﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Emgu.CV;
using Emgu.CV.Structure;
using MathNet;
using MathNet.Numerics.LinearAlgebra;
using MathNet.Numerics.LinearAlgebra.Double;
using TP.Properties;

namespace TP
{
	class GestionCam : IDisposable


	{
		
		private Image<Bgr, Byte> imageGeneral; //imageNIR afficahe
		decimal HminG = 0, HmaxG = 0, LminG = 0, LmaxG = 0; // dimension graine
		List<Recipient> ListeRecipientMatrice = new List<Recipient>(); //liste recipient de la matice, liste des cellules d'un plateau
		Rectangle Rinit;
		typeGraine typeG = new typeGraine();
		bool disposed = false;
		List<Recipient> listeContenant = new List<Recipient>();



		/* ------------------------------------------------------------------------------------------------
 *  Initialise le programme 
 *  -----------------------------------------------------------------------------------------------------*/
		public GestionCam(List<Recipient> _ListeRecipientMatrice, Rectangle rinit, decimal hminG, decimal hmaxG, decimal lminG, decimal lmaxG)
		{
			HminG = hminG; HmaxG = hmaxG; LminG = lminG; LmaxG = lmaxG;
			ListeRecipientMatrice = _ListeRecipientMatrice;
			Rinit = rinit;

		}
		public GestionCam(List<Recipient> _ListeRecipientMatrice, Rectangle rinit, typeGraine t)
		{
			HminG = (decimal)t.hauteurMin; HmaxG = (decimal)t.hauteurMax; LminG = (decimal)t.largeurMin; LmaxG = (decimal)t.largeurMax;
			ListeRecipientMatrice = _ListeRecipientMatrice;
			typeG = t;
			Rinit = rinit;
		}

		/* -----------------------------------------------------------------------------------------------------
		 *  Initialise le programme 
		 *  -----------------------------------------------------------------------------------------------------*/
		public void RepeterGestionCam( List<Recipient> _ListeRecipientMatrice, Rectangle rinit, decimal hminG, decimal hmaxG, decimal lminG, decimal lmaxG)
		{
			HminG = hminG; HmaxG = hmaxG; LminG = lminG; LmaxG = lmaxG;
			ListeRecipientMatrice = _ListeRecipientMatrice;
			Rinit = rinit;

		}
		
		/* -----------------------------------------------------------------------------------------------------
		 *  Copie de la matrice originelle
		 *  -----------------------------------------------------------------------------------------------------*/
		public List<Recipient> CopierMatrice(List<Recipient> _ListeRecipientMatrice)
		{
			List<Recipient> copieListeRecipientMatrice = new List<Recipient>();
			foreach (Recipient r in _ListeRecipientMatrice)
				copieListeRecipientMatrice.Add(new Recipient(r));
			return copieListeRecipientMatrice;
		}


		public void changerRinit( Rectangle rinit)
		{

			Rinit = rinit;

		}

		/* -----------------------------------------------------------------------------------------------------
		 *  Copie de la liste originelle
		*  -----------------------------------------------------------------------------------------------------*/
		public List<Recipient> CopierListe()
		{
			List<Recipient> copieListeRecipient = new List<Recipient>();
			foreach (Recipient r in listeContenant)
				copieListeRecipient.Add(new Recipient(r));
			return copieListeRecipient;
		}
		
		/* -----------------------------------------------------------------------------------------------------
		 *  Corrige l'image pour ajuster l'intensite en fonction de la distance par rapport au centre de l'image
		 * permet de corriger la non uniformité de l'illumination
		*  -----------------------------------------------------------------------------------------------------*/
		
		public Image<Bgr, byte> corrigerIntensiteImage(Image<Bgr, Byte> image, Rectangle ROIImage, Rectangle ROIACorriger)
		{

			int midX, midY;
			int i, j; // compteurs
			double valeur; //valeur du niveau de gris en un point
			double Rmax, Rxy, Fmax, Fxy;
			
			


			midX = ROIImage.Width / 2;
			midY = ROIImage.Height / 2;
			
			Rmax = Math.sqrt(midx*midx + midy*midY);
			Fmax = Settings.Default.Imax/Settings.Default.Imin; // Valeur max au centre et min a l'exterieur, illumination non uniforme + fort au centre 


			try
			{

                for (i = ROIACorriger.Y; i < ROIACorriger.Y + ROIACorriger.Height; i++)
                //for (i = ROIGrille.Y - ROIGrille.Height/2; i < ROIGrille.Y + ROIGrille.Height + ROIGrille.Height/2; i++)
				{
                    //for (j = ROIGrille.X; j < ROIGrille.X + ROIGrille.Width; j++)
                    for (j = 0; j < ROIImage.Width; j++)	
					{
                        if ((i >= 0) && (i < image.Height) && (j >= 0) && (j < image.Width))
                        {
                            rxy = Math.sqrt((i - midY) * (i - midY) + (j - midX) * (j - midX));
                            Fxy = (Rxy/Rmax)*(Fmax-1.0) + 1.0;                    
                            valeur = Fxy * image.Data[i, j, 0];
							
							if (valeur > 255) // saturation 
                                valeur = 255.0;
 
                            image.Data[i, j, 0] = (byte)valeur;
                            image.Data[i, j, 1] = (byte)valeur;
                            image.Data[i, j, 2] = (byte)valeur;
                        }

					}

				}
			}
			catch (Exception err)
			{

                MessageBox.Show("Impossible de corriger l'intensite de l'image : \n" + err.Message,
                "Erreur !",
                MessageBoxButtons.OK,
                MessageBoxIcon.Exclamation,
                MessageBoxDefaultButton.Button1);
			}

			return image;

		}

		public Image<Bgr, byte> corrigerImage(Image<Bgr, Byte> image, Rectangle ROIImage, Rectangle ROIACorriger)
		{

			int midX, midY;



			midX = ROIImage.Width / 2;
			midY = ROIImage.Height / 2;


			Image<Bgr, Byte> imageDeforme = image.CopyBlank();
			imageDeforme.Bitmap = image.Bitmap;

			int i, j; // compteurs
			double valeur; //valeur du niveau de gris en un point
			double r2, fr;
			double xd, yd;


			double[] vecSoln = new double[2];

			try
			{
                vecSoln[0] = Settings.Default.facteur1; // 18/04 -3.5040526712117047 X 10-8
                vecSoln[1] = Settings.Default.facteur2; // 18/04 7.72543622516225 X 10-15
			}
			catch (Exception)
			{
				vecSoln[0] = Settings.Default.facteurDefault1;
				vecSoln[1] = Settings.Default.facteurDefault2;
			}

			try
			{

                for (i = ROIACorriger.Y; i < ROIACorriger.Y + ROIACorriger.Height; i++)
                //for (i = ROIGrille.Y - ROIGrille.Height/2; i < ROIGrille.Y + ROIGrille.Height + ROIGrille.Height/2; i++)
				{
                    //for (j = ROIGrille.X; j < ROIGrille.X + ROIGrille.Width; j++)
                    for (j = 0; j < ROIImage.Width; j++)	
					{
                        if ((i >= 0) && (i < image.Height) && (j >= 0) && (j < image.Width))
                        {
                            //r = Math.Pow((i - midY) * (i - midY) + (j - midX) * (j - midX), 0.5);
                            //fr = 1.0 + vecSoln[0] * r * r + vecSoln[1] * r * r * r * r;
                            r2 = (i - midY) * (i - midY) + (j - midX) * (j - midX);
                            fr = 1.0 + vecSoln[0] * r2 + vecSoln[1] * r2 * r2;
                            xd = (j - midX) * fr + midX;
                            yd = (i - midY) * fr + midY;

                            if ((xd >= 0) && (xd < imageDeforme.Width) && (yd >= 0) && (yd < imageDeforme.Height))
                                valeur = imageDeforme.Data[(int)yd, (int)xd, 0];
                            else
                                valeur = 0;
                            image.Data[i, j, 0] = (byte)valeur;
                            image.Data[i, j, 1] = (byte)valeur;
                            image.Data[i, j, 2] = (byte)valeur;
                        }

					}

				}
			}
			catch (Exception err)
			{

                MessageBox.Show("Impossible de corriger l'image : \n" + err.Message,
                "Erreur !",
                MessageBoxButtons.OK,
                MessageBoxIcon.Exclamation,
                MessageBoxDefaultButton.Button1);
			}

			return image;

		}





        public static Image<Bgr, byte> corrigerImageS(Image<Bgr, Byte> image, Rectangle ROIImage, Rectangle ROIGrille)
        {

            int midX, midY;



            midX = ROIImage.Width / 2;
            midY = ROIImage.Height / 2;


            Image<Bgr, Byte> imageDeforme = image.CopyBlank();
            imageDeforme.Bitmap = image.Bitmap;

            int i, j; // compteurs
            double valeur; //valeur du niveau de gris en un point
            double  fr,r2;
            double xd, yd;


            double[] vecSoln = new double[2];

            try
            {
                vecSoln[0] = Settings.Default.facteur1;// 18/04 -3.5040526712117047 X 10-8
                vecSoln[1] = Settings.Default.facteur2; // 18/04 7.72543622516225 X 10-15
            }
            catch (Exception)
            {
                vecSoln[0] = Settings.Default.facteurDefault1;
                vecSoln[1] = Settings.Default.facteurDefault2;
            }

            try
            {
                for (j = ROIGrille.X; j < ROIGrille.X + ROIGrille.Width; j++)
                {

                    for (i = ROIGrille.Y; i < ROIGrille.Y + ROIGrille.Height; i++)
                    {
                        r2 = (i - midY) * (i - midY) + (j - midX) * (j - midX);
                        fr = 1.0 + vecSoln[0] * r2 + vecSoln[1] * r2 * r2;
                        
                  
                        xd = (j - midX) * fr + midX;
                        yd = (i - midY) * fr + midY;


                        if ((xd >= 0) && (xd < imageDeforme.Width) && (yd >= 0) && (yd < imageDeforme.Height))
                            valeur = imageDeforme.Data[(int)yd, (int)xd, 0];
                        else
                            valeur = 0;
                        image.Data[i, j, 0] = (byte)valeur;
                        image.Data[i, j, 1] = (byte)valeur;
                        image.Data[i, j, 2] = (byte)valeur;

                    }

                }
            }
            catch (Exception err)
            {

                MessageBox.Show("Impossible de corriger l'image : \n" + err.Message,
                "Erreur !",
                MessageBoxButtons.OK,
                MessageBoxIcon.Exclamation,
                MessageBoxDefaultButton.Button1);
            }

            return image;

        }




		/* -----------------------------------------------------------------------------------------------------
				 *  traitement de l'imageNIR pour un total de x iterations formant l'imageNIR complete
				 *  -----------------------------------------------------------------------------------------------------*/
		public int Traitement(Image<Bgr, Byte> image, /*Image<Bgr, Byte> image2,*/ Image<Bgr, Byte> matrice, int NoPLateau, int nombreCellule)
		{
			listeContenant.Clear();
			listeContenant = CopierMatrice(ListeRecipientMatrice);
			listeContenant.Sort();
			int[] position = new int[4] { 1, 1, 1, 1 };

			Rectangle roiGrille = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height);

			for (int hauteur = 0; hauteur < Settings.Default.hauteurMatrice; hauteur++)
			{
			 
				for (int largeur = 0; largeur < Settings.Default.largeurMatrice; largeur++)
				{
					int i = (Settings.Default.largeurMatrice * hauteur) + largeur;
					Rectangle r = new Rectangle(image.ROI.X + listeContenant[i].ZoneROI.X, image.ROI.Y + listeContenant[i].ZoneROI.Y, listeContenant[i].ZoneROI.Width, listeContenant[i].ZoneROI.Height);
					image.ROI = r;
					//VecteurCaracteristiqueImage vec = new VecteurCaracteristiqueImage(imageNIR.Copy());
					/*envoie nouvelle imageNIR à vecteur caractéristique*/
					VecteurCaracteristiqueImage vec = new VecteurCaracteristiqueImage(image.Copy(), typeG);
					listeContenant[i].Moyenne = vec.DetecterMoyenne();
					
					listeContenant[i].ListeGraine = vec.DetecterGraine(listeContenant[i].Moyenne[1]); /*passe vecteur de moyenne*/
				  
					
					// imageNIR.ROI = Rinit;
					image.ROI = roiGrille;
				}
			}
		   /* for (int hauteur = 0; hauteur < Settings.Default.hauteurMatrice; hauteur++)
				for (int largeur = 0; largeur < Settings.Default.largeurMatrice; largeur++)
					listeContenant[(Settings.Default.largeurMatrice * hauteur) + largeur].moyenneGenerale(position[hauteur]);*/

			int nombre = 48;
 
			return nombre;
		}


		public int Traitement3(Image<Bgr, Byte> imageNir, Image<Bgr, Byte> imageUv, Image<Bgr, Byte> matrice, int NoPLateau, int nombreCellule)
		{
			listeContenant.Clear();
			listeContenant = CopierMatrice(ListeRecipientMatrice);
			listeContenant.Sort();

			Rectangle roiGrille = new Rectangle(imageNir.ROI.X, imageNir.ROI.Y, imageNir.ROI.Width, imageNir.ROI.Height);

			for (int hauteur = 0; hauteur < Settings.Default.hauteurMatrice; hauteur++)
			{

				for (int largeur = 0; largeur < Settings.Default.largeurMatrice; largeur++)
				{
					int i = (Settings.Default.largeurMatrice * hauteur) + largeur;
					Rectangle r = new Rectangle(imageNir.ROI.X + listeContenant[i].ZoneROI.X, imageNir.ROI.Y + listeContenant[i].ZoneROI.Y, listeContenant[i].ZoneROI.Width, listeContenant[i].ZoneROI.Height);
					imageNir.ROI = r;
					imageUv.ROI = r;

					VecteurCaracteristiqueImage vec = new VecteurCaracteristiqueImage(imageNir.Copy(), imageUv.Copy(), typeG);

					listeContenant[i].ListeGraine = vec.DetecterGraine3();

					imageNir.ROI = roiGrille;
				}
			}

			int nombre = 48;

			return nombre;
		}



		/* -----------------------------------------------------------------------------------------------------
		 *  traitement de l'imageNIR pour un total de x iterations formant l'imageNIR complete
		 *  -----------------------------------------------------------------------------------------------------*/
		public int Traitement2(Image<Bgr, Byte> imageNIR, Image<Bgr, Byte> matrice, int NoPLateau,int nombreCellule)
		{
			listeContenant.Clear();
			listeContenant = CopierMatrice(ListeRecipientMatrice);
			listeContenant.Sort();
			int[] position = new int[4] { 1, 1, 1, 1 };

			Rectangle roiGrille = new Rectangle(imageNIR.ROI.X, imageNIR.ROI.Y, imageNIR.ROI.Width, imageNIR.ROI.Height);

			for (int hauteur = 0; hauteur < Settings.Default.hauteurMatrice; hauteur++)
			{
				int[] moyenneLigne = new int[3] { 0, 0, 0 };
				int nbGraineLigne = 0;
				for (int largeur = 0; largeur < Settings.Default.largeurMatrice; largeur++)
				{
					int i = (Settings.Default.largeurMatrice * hauteur) + largeur;
					Rectangle r = new Rectangle(imageNIR.ROI.X + listeContenant[i].ZoneROI.X, imageNIR.ROI.Y + listeContenant[i].ZoneROI.Y, listeContenant[i].ZoneROI.Width, listeContenant[i].ZoneROI.Height);
					imageNIR.ROI = r;
					//VecteurCaracteristiqueImage vec = new VecteurCaracteristiqueImage(imageNIR.Copy());
					VecteurCaracteristiqueImage vec = new VecteurCaracteristiqueImage(imageNIR.Copy(), typeG);
					listeContenant[i].Moyenne = vec.DetecterMoyenne();
					if (listeContenant[i].Position == 1)
					{
						//listeContenant[i].ListeGraine = vec.DetecterGraine(listeContenant[i].Moyenne[1], HminG, HmaxG, LminG, LmaxG);
						listeContenant[i].ListeGraine = vec.DetecterGraine(listeContenant[i].Moyenne[1]);
					}
					moyenneLigne[listeContenant[i].Position]++;
					nbGraineLigne += listeContenant[i].NombreGraine;
				   // imageNIR.ROI = Rinit;
					imageNIR.ROI = roiGrille;
				}
				position[hauteur] = moyenneLigne[0] >= 3 || (moyenneLigne[0] != 0 && nbGraineLigne == 0) ? 0 : moyenneLigne[2] >= 3 || (moyenneLigne[2] != 0 && nbGraineLigne == 0) ? 2 : 1;
			
			}
			for (int hauteur = 0; hauteur < Settings.Default.hauteurMatrice; hauteur++)
				for (int largeur = 0; largeur < Settings.Default.largeurMatrice; largeur++)
					listeContenant[(Settings.Default.largeurMatrice * hauteur) + largeur].moyenneGenerale(position[hauteur]);

			int nombre = 0;
			foreach (Recipient c in listeContenant)
			{
				if (c.Position == 1)
					nombre++;
			}
			return nombre;
		}

 /* ----------------------------------------------------------------------------------------------------------
 *  Fonction calculant la variance des pixels de spheigne imageNIR Gray
 *  ---------------------------------------------------------------------------------------------------------*/
		public double DetecterStdDifference(Image<Gray, Byte> diff, double moyenne)
		{
			Image<Gray, Byte> diffGray = diff.Copy().PyrDown().PyrUp();

			double var;
			double somme = 0;

			for (int i = 0; i < diffGray.Height; i++)
				for (int j = 0; j < diffGray.Width; j++)
				{
						somme = somme + ((diffGray.Data[i, j, 0] - moyenne) * (diffGray.Data[i, j, 0] - moyenne));

				}
			var = somme / (double)(diffGray.Width * diffGray.Height);

			return Math.Sqrt(var);
		}

		/* ----------------------------------------------------------------------------------------------------------
			   *  Fonction calculant la moyenne des pixels correspondant a de la spheigne Image Gray
			   *  ---------------------------------------------------------------------------------------------------------*/
		public double DetecterMoyenneDifference(Image<Gray, Byte> diff)
		{
			Image<Gray, Byte> diffGray = diff.Copy().PyrDown().PyrUp();

			double moy;
			double somme = 0;

			for (int i = 0; i < diffGray.Height; i++)
				for (int j = 0; j < diffGray.Width; j++)
				{
						somme = somme + diffGray.Data[i, j,0];
					   
				}


			moy = somme / (double)(diffGray.Width * diffGray.Height);

			return moy;
		}

		/* ----------------------------------------------------------------------------------------------------------
		 *  Fonction calculant la variance des pixels de spheigne Image Bgr
		 *  ---------------------------------------------------------------------------------------------------------*/
		public double DetecterStdDifference(Image<Bgr, Byte> diff, double moyenne)
		{
			Image<Bgr, Byte> diffBgr = diff.Copy().PyrDown().PyrUp();

			double var;
			double somme = 0;

			for (int i = 0; i < diffBgr.Height; i++)
				for (int j = 0; j < diffBgr.Width; j++)
				{
					somme = somme + ((diffBgr.Data[i, j, 0] - moyenne) * (diffBgr.Data[i, j, 0] - moyenne));

				}
			var = somme / (double)(diffBgr.Width * diffBgr.Height);

			return Math.Sqrt(var);
		}

		/* ----------------------------------------------------------------------------------------------------------
			   *  Fonction calculant la moyenne des pixels correspondant a de la spheigne Bgr
			   *  ---------------------------------------------------------------------------------------------------------*/
		public double DetecterMoyenneDifference(Image<Bgr, Byte> diff)
		{
			Image<Bgr, Byte> diffBgr = diff.Copy().PyrDown().PyrUp();

			double moy;
			double somme = 0;

			for (int i = 0; i < diffBgr.Height; i++)
				for (int j = 0; j < diffBgr.Width; j++)
				{
					somme = somme + diffBgr.Data[i, j, 0];

				}


			moy = somme / (double)(diffBgr.Width * diffBgr.Height);

			return moy;
		}



	   /* -----------------------------------------------------------------------------------------------------
		   Positionner la grille sur les 4 premieres rangees de cellules d'un plateau
		   ---------------------------------------------------------------------------------------------------*/
		public Rectangle PositionnerGrille(Image<Bgr, Byte> image, Image<Bgr, Byte> back)
		{               
		   
			Rectangle r = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height);
			image.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight);
            back.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight);

			Image<Bgr, Byte> Difference = image.AbsDiff(back);


		   // double moy = DetecterMoyenneDifference(Difference);
		   // double std = DetecterStdDifference(Difference, moy);
	  
			Image<Bgr, Byte> DifferenceT  = Difference.ThresholdBinary(new Bgr(Settings.Default.SeuilDebutPlateau, Settings.Default.SeuilDebutPlateau, Settings.Default.SeuilDebutPlateau), new Bgr(255, 255, 255)); // 30 choisi apres analyse de l histo de l imageNIR Difference

		   
			//DifferenceT = DifferenceT.Erode(4).Dilate(4);
			//DifferenceT = DifferenceT.Dilate(4).Erode(4);

   
			using (MemStorage storage = new MemStorage()) //allocate storage for contour approximation
			{
				using (Image<Gray, Byte> DifferenceTG = DifferenceT.Convert<Gray, byte>()) //allocate storage for contour approximation, using (Image<Gray, Byte> DifferenceTG = DifferenceT.Convert<Gray,byte>())
				{
					
					//detect the contours and loop through each of them
					for (Contour<Point> contours = DifferenceTG.FindContours(
						   Emgu.CV.CvEnum.CHAIN_APPROX_METHOD.CV_CHAIN_APPROX_SIMPLE,
						   Emgu.CV.CvEnum.RETR_TYPE.CV_RETR_LIST,
						   storage);
						contours != null;
						contours = contours.HNext)
					{

						//if (((contours.BoundingRectangle.Width * contours.BoundingRectangle.Height) > (r.Width * r.Height)) && ((contours.BoundingRectangle.Y > 0) && (contours.BoundingRectangle.Y < 100)) && (contours.BoundingRectangle.Width >= r.Width))
                    //    if (((contours.BoundingRectangle.Width * contours.BoundingRectangle.Height) > (r.Width * r.Height)) && ((contours.BoundingRectangle.Y > 0) && (contours.BoundingRectangle.Y < 500)))
                      //  {
                        //if ((contours.BoundingRectangle.Width * contours.BoundingRectangle.Height) > (r.Width * r.Height))
                 /*       if ((contours.BoundingRectangle.Height >= r.Height) && (contours.BoundingRectangle.Y < 100) && (contours.BoundingRectangle.X < 100))
                        {
							image.ROI = new Rectangle(r.X, contours.BoundingRectangle.Bottom - r.Height, r.Width, r.Height); // avant 2804 -100
                            
							//Rectangle rDraw = CvInvoke.cvBoundingRect(contours, 1);
							//image.Draw(rDraw, new Bgr(Color.Red), 10);
                            //image.Draw(image.ROI, new Bgr(Color.Green), 10);

                             //image.ROI = new Rectangle(r.X, r.Y, r.Width, r.Height); // a ajuster quand le positionnement sera ok
                            return image.ROI;
						}*/

                        if (((contours.BoundingRectangle.Width * contours.BoundingRectangle.Height) > (r.Width * r.Height)) && ((contours.BoundingRectangle.Y < 100) && (contours.BoundingRectangle.X < 100)))
                        {

                            image.ROI = new Rectangle(r.X, contours.BoundingRectangle.Bottom - r.Height, r.Width, r.Height); 
                            return image.ROI;  //AENLEVER

                /*            Rectangle roiAAjuster = new Rectangle(r.X, contours.BoundingRectangle.Bottom - r.Height, r.Width, r.Height); // avant 2804 -100
                            
                            
                            // Rectangle rDraw = CvInvoke.cvBoundingRect(contours, 1);
                            // image.Draw(rDraw, new Bgr(Color.Red), 6);
                            //image.ROI = new Rectangle(r.X, contours.BoundingRectangle.Bottom - r.Height, r.Width, r.Height); // avant 2804 -100

                            //image.ROI = new Rectangle(r.X, contours.BoundingRectangle.Bottom - r.Height  - r.Height / 2, r.Width, r.Height); // avant 2804 -100
                            Rectangle rDetecterLigne =  new Rectangle(r.X, contours.BoundingRectangle.Bottom - r.Height - r.Height / 2, r.Width, r.Height);
                            List<LineSegment2D> lines = DetecterLignes.CalculerHorizontales(image.Convert<Gray, byte>(), rDetecterLigne);


                            try
                            {


                                int upper = (from l in lines
                                             where (rDetecterLigne.Y + l.P1.Y) < roiAAjuster.Y
                                             select l.P1.Y).Max();

                                r.Y = rDetecterLigne.Y + upper;
                                image.ROI = r;
                                return image.ROI;
                            }
                            catch (Exception)
                            {
                                image.ROI = r;
                                return image.ROI;
                            } */


                        }

					}
				}
			}

            image.ROI = r;
			return image.ROI;
		
		}


		/* -----------------------------------------------------------------------------------------------------
	   Positionner la grille sur les 4 premieres rangees de cellules d'un plateau en maximisant le cv de la projection
		 horizontale
	   ---------------------------------------------------------------------------------------------------*/
		public Rectangle PositionnerGrilleMAXcv(Image<Bgr, Byte> image, Image<Bgr, Byte> back)
		{

			Rectangle r = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height);
			image.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight);

			Image<Gray, byte> imageG = image.Convert<Gray, byte>();
			Image<Gray, byte> backG = back.Convert<Gray, byte>();


			Image<Gray, Byte> Difference = imageG.AbsDiff(backG);

			int nbPix = 0;
			double mean = 0;
			double somme = 0.0;
			double std, cv, cvAvant;

			cvAvant = cv = 0.0;

			for (int pY = r.Height; pY < image.Height; pY++)
			{

				cvAvant = cv;
				nbPix = 0;
				mean = 0;


				// calculer la moyenne de la region centree a py

				for (int i = image.ROI.X; i < image.ROI.X + image.ROI.Width; i++)
				{
					for (int j = pY - 5 ; j < pY + 5; j++)
					{
						mean += Difference.Data[j, i, 0];
						nbPix++;
					}
				}

				mean /= nbPix;
				somme = 0.0;

				for (int i = image.ROI.X; i < image.ROI.X + image.ROI.Width; i++)
				{
					for (int j = pY - 5; j < pY + 5; j++)
					{
						somme = somme + ((Difference.Data[i, j, 0] - mean) * (Difference.Data[i, j, 0] - mean));

					}
				}
				std = Math.Sqrt(somme / (double)  nbPix);

				cv = std / mean;

				if (cv > 1.25 * cvAvant)
				{
					image.ROI = new Rectangle(r.X, pY-r.Height - 40, r.Width, r.Height);
					return image.ROI;
				}


			}

			image.ROI = new Rectangle(r.X, r.Y, r.Width, r.Height);
			return image.ROI;

		}


		/* -----------------------------------------------------------------------------------------------------
				  Ajuster la grille en calculant le deplacement moyen entre les Bounding Box de l'imageNIR de 
		  difference et les ROI de chaque cellule de la grille
				  ---------------------------------------------------------------------------------------------------*/
		public Rectangle AjusterGrilleOptimale(Image<Bgr, Byte> image, Image<Bgr, Byte> back, Image<Bgr, Byte> template)
		{

			Rectangle r = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height);
			image.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight);

			Recipient c = ListeRecipientMatrice[0];

			Image<Bgr, Byte> Difference = image.AbsDiff(back);

			// double moy = DetecterMoyenneDifference(Difference);
			//  double std = DetecterStdDifference(Difference, moy);

			Image<Bgr, Byte> DifferenceT = Difference.ThresholdBinary(new Bgr(30, 30, 30), new Bgr(255, 255, 255)); // 30 choisi apres analyse de l histo de l imageNIR Difference


			DifferenceT = DifferenceT.Erode(4).Dilate(4);
			DifferenceT = DifferenceT.Dilate(4).Erode(4);

			DifferenceT.Not(); // inversion de l'imageNIR

			int nbCellule = 0;
			int dx = 0;
			int dy = 0;

			using (MemStorage storage = new MemStorage()) //allocate storage for contour approximation
			{
				using (Image<Gray, Byte> DifferenceTG = DifferenceT.Convert<Gray, byte>()) //allocate storage for contour approximation, using (Image<Gray, Byte> DifferenceTG = DifferenceT.Convert<Gray,byte>())
				{

					//detect the contours and loop through each of them
					for (Contour<Point> contours = DifferenceTG.FindContours(
						   Emgu.CV.CvEnum.CHAIN_APPROX_METHOD.CV_CHAIN_APPROX_SIMPLE,
						   Emgu.CV.CvEnum.RETR_TYPE.CV_RETR_LIST,
						   storage);
						contours != null;
						contours = contours.HNext)
					{
						if ((contours.BoundingRectangle.Y >= (r.Y - r.Height / 8)) && (contours.BoundingRectangle.Y < (r.Y + r.Height / 8)))
						{

							if (((contours.BoundingRectangle.Width * contours.BoundingRectangle.Height) > (c.ZoneROI.Width * c.ZoneROI.Height * 0.8)) && ((contours.BoundingRectangle.Width * contours.BoundingRectangle.Height) <= (c.ZoneROI.Width * c.ZoneROI.Height)))
							{
								// trouver ROI de la cellule la plus proche
								int distMin = 9999999;
								int dxMin = 999999;
								int dyMin = 99999;

								nbCellule++;

								int dist = 0;
								for (int i = 0; i < Settings.Default.largeurMatrice * Settings.Default.hauteurMatrice; i++)
								{
									c = ListeRecipientMatrice[i];
									dist = Math.Abs(contours.BoundingRectangle.X - r.X + c.ZoneROI.X+10) + Math.Abs(contours.BoundingRectangle.Y - r.Y + c.ZoneROI.Y+10);
									if (dist < distMin)
									{
										distMin = dist;
										dxMin = (contours.BoundingRectangle.X - r.X + c.ZoneROI.X+10);
										dyMin = (contours.BoundingRectangle.Y - r.Y + c.ZoneROI.Y+10);
									}


								}
								dx += dxMin;
								dy += dyMin;
							}
						}

					}
				}
			}

			if (nbCellule > 0)
			{
				dx /= nbCellule;
				dy /= nbCellule;
				image.ROI = new Rectangle(r.X + dx, r.Y + dy, r.Width, r.Height);
				return image.ROI;
			}

			return r;

		}

		/* -----------------------------------------------------------------------------------------------------
				Positionner FINEMENT la grille sur les 4 premieres rangees de cellules d'un plateau
				---------------------------------------------------------------------------------------------------*/
		public Rectangle AjusterGrilleVerHor(Image<Bgr, Byte> image)
		{

			Rectangle r = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height);
			image.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight);

			Image<Gray, Byte> im = image.Convert<Gray, Byte>();

			int minMean = 256;
			int nbPix;
			int minDec = 0;
			int mean;

			Recipient c = ListeRecipientMatrice[0]; // premiere cellule en haut a gauche de la grille 


// Ajuster verticalement
			for (int decY = -c.ZoneROI.Height / 4; decY < c.ZoneROI.Height / 4; decY++) //20
			{

				nbPix = 0;
				mean = 0;

				// calculer la moyenne partie haute

				for (int i = c.ZoneROI.X + r.X; i < c.ZoneROI.Width + c.ZoneROI.X + r.X; i++)
				{
					for (int j = decY + c.ZoneROI.Y + r.Y; j < decY + c.ZoneROI.Y + 10 + r.Y; j++)
					{
						if((j > 0) && (j < image.ROI.Height))
						{
							mean += im.Data[j, i, 0];
							nbPix++;
						}
					}
				}

				// calculer la partie basse

				for (int i = c.ZoneROI.X + r.X; i < c.ZoneROI.Width + c.ZoneROI.X + r.X; i++)
				{
					for (int j = decY + c.ZoneROI.Height + c.ZoneROI.Y - 10 + r.Y; j < decY + c.ZoneROI.Height + c.ZoneROI.Y + r.Y; j++)
					{
						if ((j > 0) && (j < image.ROI.Height))
						{
							mean += im.Data[j, i, 0];
							nbPix++;
						}
					}
				}

				mean /= nbPix;

				if (mean < minMean)
				{
					minMean = mean;
					minDec = decY;
				}

			}

			r.Y += minDec;
			if (r.Y + r.Height > im.ROI.Height) // empecher que la grille depasse en bas de l'imageNIR
				r.Y = im.ROI.Height - r.Height;


			minMean = 256;
			nbPix = 0;
			minDec = 0;
			mean = 0;

			// Ajuster horizontalement
			for (int decX = -c.ZoneROI.Width / 4; decX < c.ZoneROI.Width / 4; decX++) //20
			{

				nbPix = 0;
				mean = 0;

				// calculer la moyenne partie gauche

				for (int i = decX + c.ZoneROI.X + r.X; i < decX + c.ZoneROI.X + r.X + 10; i++)
				{
					for (int j = c.ZoneROI.Y + r.Y; j < c.ZoneROI.Height + c.ZoneROI.Y + r.Y; j++)
					{
						if ((i > 0) && (i < image.ROI.Width))
						{
							mean += im.Data[j, i, 0];
							nbPix++;
						}
					}
				}

				// calculer la partie droite

				for (int i = decX + c.ZoneROI.Width + c.ZoneROI.X + r.X - 10; i < decX + c.ZoneROI.Width + c.ZoneROI.X + r.X; i++)
				{
					for (int j = c.ZoneROI.Y + r.Y; j < c.ZoneROI.Y + c.ZoneROI.Height + r.Y; j++)
					{
						if ((i > 0) && (i < image.ROI.Width))
						{
							mean += im.Data[j, i, 0];
							nbPix++;
						}
					}
				}

				mean /= nbPix;

				if (mean < minMean)
				{
					minMean = mean;
					minDec = decX;
				}

			}

			r.X += minDec;
			if (r.X + r.Width > im.ROI.Width) // empecher que la grille depasse a droite de l'imageNIR
				r.X = im.ROI.Width - r.Width;

			return r;

		}


		/* -----------------------------------------------------------------------------------------------------
			  Positionner FINEMENT la grille sur les 4 premieres rangees de cellules d'un plateau
			  ---------------------------------------------------------------------------------------------------*/
		public Rectangle AjusterGrille(Image<Bgr, Byte> image)
		{

			Rectangle r = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height); /*remettre rectangle sur ligne la plus près dans l'image*/
			image.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight);

			Image<Gray, Byte> im = image.Convert<Gray, Byte>();

			int minMean = 256;
			int nbPix;
			int minDec = 0;
			int mean;

			Recipient c = ListeRecipientMatrice[0];



			for (int decY = 0; decY < 20; decY++) //20
			{

				nbPix = 0;
				mean = 0;

				// calculer la moyenne partie haute

				for (int i = c.ZoneROI.X + r.X; i < c.ZoneROI.Right + r.X; i++)
				{
					for (int j = decY + c.ZoneROI.Y + r.Y; j < decY + c.ZoneROI.Y + 10 + r.Y; j++)
					{
						mean += im.Data[j, i, 0];
						nbPix++;
					}
				}

				// calculer la partie basse

				for (int i = c.ZoneROI.X + r.X; i < c.ZoneROI.Right + r.X; i++)
				{
					for (int j = decY + c.ZoneROI.Bottom - 10 + r.Y; j < decY + c.ZoneROI.Bottom + r.Y; j++)
					{
						mean += im.Data[j, i, 0];
						nbPix++;
					}
				}

				mean /= nbPix;

				if (mean < minMean)
				{
					minMean = mean;
					minDec = decY;
				}

			}

			r.Y += minDec;


			if (r.Y + r.Height > im.ROI.Height) // empecher que la grille depasse en bas de l'imageNIR
				r.Y = im.ROI.Height - r.Height;

			return r;

		}

        public Rectangle AjusterGrilleIter0(Image<Bgr, Byte> image)
        {

            Rectangle r = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height); // ROI Grille
            image.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight); // ROI image complete


            //image.ROI = new Rectangle(r.X, r.Y - r.Height / 2, r.Width , r.Height + r.Height/2);
            Rectangle rDetecterLigne = new Rectangle(r.X, r.Y - r.Height / 2, r.Width, r.Height + r.Height / 2);
            
            List<LineSegment2D> lines = DetecterLignes.CalculerHorizontales(image.Convert<Gray, byte>(), rDetecterLigne);

            
            try
            {

   
                int upper = (from l in lines
                             where (image.ROI.Y + l.P1.Y) < r.Y
                             select l.P1.Y).Max();

                r.Y = image.ROI.Y + upper;
                image.ROI = r;           
                return r;
            }
            catch (Exception)
            {
                image.ROI = r;
                return r;
            }

        }

		public Rectangle AjusterGrille2(Image<Bgr, Byte> image, int iter)
		{

			Rectangle r = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height); // ROI Grille
            Rectangle rDetecterLigne = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height);
		    image.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight); // ROI image complete

            if (iter == 0)
            {
                rDetecterLigne.Y = r.Y - r.Height / 2;
            }
            else
            {
                rDetecterLigne.Y = r.Bottom - r.Height / 2;
            }
                
     
            List<LineSegment2D> lines = DetecterLignes.CalculerHorizontales(image.Convert<Gray, byte>(), rDetecterLigne);


			try
			{
                if (iter > 0)
                {

                   int lower = (from l in lines
                                where (rDetecterLigne.Y + l.P1.Y) > r.Bottom
                               select l.P1.Y).Min();


                   r.Y = rDetecterLigne.Y + lower - r.Height;                
                }

                //int closest = lines.Aggregate((a, b) => Math.Abs(image.ROI.Y + a.P1.Y - r.Y) < Math.Abs(image.ROI.Y + b.P1.Y - r.Y) ? a : b).P1.Y;
                //r.Y = image.ROI.Y + closest;

                if (iter == 0)
                {

                  int upper = (from l in lines
                               where (rDetecterLigne.Y + l.P1.Y) < r.Y
                                 select l.P1.Y).Max();

                  r.Y = rDetecterLigne.Y + upper;
                }

                image.ROI = r;
				return r;
			}
			catch (Exception)
			{
                image.ROI = r;
				return r;
			}

		}


		/* -----------------------------------------------------------------------------------------------------
			  Positionner FINEMENT la grille sur les 4 premieres rangees de cellules d'un plateau
			  ---------------------------------------------------------------------------------------------------*/
		public Rectangle AjusterGrilleVersLeHaut(Image<Bgr, Byte> image)
		{

			Rectangle r = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height);
			image.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight);

			Image<Gray, Byte> im = image.Convert<Gray, Byte>();

			int minMean = 256;
			int nbPix;
			int minDec = 0;
			int mean;

			Recipient c = ListeRecipientMatrice[36];



			for (int decY = 0; decY < 20; decY++)
			{

				nbPix = 0;
				mean = 0;

				// calculer la moyenne partie haute

				for (int i = c.ZoneROI.X + r.X; i < c.ZoneROI.Right + r.X; i++)
				{
					for (int j =  c.ZoneROI.Y + r.Y - decY; j < c.ZoneROI.Y + 10 + r.Y - decY; j++)
					{
						mean += im.Data[j, i, 0];
						nbPix++;
					}
				}


				mean /= nbPix;

				if (mean < minMean)
				{
					minMean = mean;
					minDec = decY;
				}

			}

			r.Y -= minDec;


			if (r.Y < 0) // empecher que la grille depasse en haut de l'imageNIR
				r.Y = 0;

			return r;

		}



		/* -----------------------------------------------------------------------------------------------------
			  Positionner FINEMENT la grille sur les 4 premieres rangees de cellules d'un plateau
			  ---------------------------------------------------------------------------------------------------*/
		public Rectangle AjusterGrilleVersLeBas(Image<Bgr, Byte> image)
		{

			Rectangle r = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height);
			image.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight);

			Image<Gray, Byte> im = image.Convert<Gray, Byte>();

			int minMean = 256;
			int nbPix;
			int minDec = 0;
			int mean;

			Recipient c = ListeRecipientMatrice[0];



			for (int decY = 0; decY < 20; decY++)
			{

				nbPix = 0;
				mean = 0;

				// calculer la moyenne partie haute

				for (int i = c.ZoneROI.X + r.X; i < c.ZoneROI.Right + r.X; i++)
				{
					for (int j = decY + c.ZoneROI.Y + r.Y; j < decY + c.ZoneROI.Y + 10 + r.Y; j++)
					{
						mean += im.Data[j, i, 0];
						nbPix++;
					}
				}

				// calculer la partie basse

				for (int i = c.ZoneROI.X + r.X; i < c.ZoneROI.Right + r.X; i++)
				{
					for (int j = decY + c.ZoneROI.Bottom - 10 + r.Y; j < decY + c.ZoneROI.Bottom + r.Y; j++)
					{
						mean += im.Data[j, i, 0];
						nbPix++;
					}
				}

				mean /= nbPix;

				if (mean < minMean)
				{
					minMean = mean;
					minDec = decY;
				}

			}

			r.Y += minDec;


			if (r.Y + r.Height > im.ROI.Height) // empecher que la grille depasse en bas de l'imageNIR
				r.Y = im.ROI.Height - r.Height;

			return r;

		}




		/* -----------------------------------------------------------------------------------------------------
			Positionner la grille sur les 4 premieres rangees de cellules d'un plateau
			---------------------------------------------------------------------------------------------------*/
		public bool GrillePositionner(Rectangle roiAvant, Rectangle roiApres)
		{

			bool grillePositionner = true;
			

		   // verifier si les ROIs des la grille a changee 
		   if ((roiAvant.X == roiApres.X)  && (roiAvant.Y == roiApres.Y) && (roiAvant.Width == roiApres.Width) && (roiAvant.Height == roiApres.Height))
		   {
			   grillePositionner = false;
			   return grillePositionner;
		   }
		   else
			   return grillePositionner;
	
		}





		/* -----------------------------------------------------------------------------------------------------
 *  Rotation de l'imageNIR
 *  -----------------------------------------------------------------------------------------------------*/
		public void listeRecipientClear()
		{

			ListeRecipientMatrice.Clear();

		}


	   // public List<Image<Bgr, Byte>> retourlisteImage  { get { return listeImage; } }

		/* -----------------------------------------------------------------------------------------------------
		 *  Rotation de l'imageNIR
		 *  -----------------------------------------------------------------------------------------------------*/
		public void imageGeneralClear()
		{

			if (imageGeneral != null)
			{
				imageGeneral.Dispose();
				imageGeneral = null;
			}

		}

		public void traitementTest()
		{
			listeContenant.Clear();
			listeContenant = CopierMatrice(ListeRecipientMatrice);
			listeContenant.Sort();

		}


		/* -----------------------------------------------------------------------------------------------------
* Fonction ajoutant dessin
*  -----------------------------------------------------------------------------------------------------*/

		public Image<Bgr, Byte> dessinerTest(Image<Bgr, Byte> im1)
		{

			foreach (Recipient c in listeContenant)
			{
	
					Point[] pts = c.contourPoint;
					im1.Draw(c.ZoneROI, new Bgr(Color.White), 6); // On trace le contour sur l'imageNIR original

			}
			return im1;
		}



	 /* -----------------------------------------------------------------------------------------------------
		* Fonction ajoutant dessin
		*  -----------------------------------------------------------------------------------------------------*/

		public Image<Bgr, Byte> dessiner(Image<Bgr, Byte> im1)
		{
		
		   Rectangle r = new Rectangle(im1.ROI.X, im1.ROI.Y, im1.ROI.Width, im1.ROI.Height);

		   Rectangle rC = new Rectangle(im1.ROI.X, im1.ROI.Y, im1.ROI.Width, im1.ROI.Height);

			try
			{
				foreach (Recipient c in listeContenant)
				{
				   // if (c.Position == 1) // voir si rangee trop brillante > 150 dans traitement graine
				  //  {
					rC.X = c.ZoneROI.X + im1.ROI.X;
					rC.Y = c.ZoneROI.Y + im1.ROI.Y;
					rC.Width = c.ZoneROI.Width;
					rC.Height = c.ZoneROI.Height;
					im1.ROI = rC;
				   for(int k=0;k< c.ListeGraine.Count;k++)
						im1.Draw(c.ListeGraine[k].Contour, new Bgr(255, 0, 255), 6);
				   im1.ROI = r;

						switch (c.NombreGraine)
						{
							case 0: // Pas de graine dans la cellule
								for (int i = c.ZoneROI.X + im1.ROI.X; i < c.ZoneROI.Right + im1.ROI.X; i++)
									for (int j = c.ZoneROI.Y + im1.ROI.Y; j < c.ZoneROI.Bottom + im1.ROI.Y; j++)
									{
										im1.Data[j, i, 2] = 255; // rouge clair
										//im1.Data[j, i, 0] = 0; im1.Data[j, i, 1] = 0; // rouge fonce
									}
								 break;
							case 1: // Une seule graine BLEU => OK
								for (int i = c.ZoneROI.X + im1.ROI.X; i < c.ZoneROI.Right + im1.ROI.X; i++)
									for (int j = c.ZoneROI.Y + im1.ROI.Y; j < c.ZoneROI.Bottom + im1.ROI.Y; j++)
									{
										im1.Data[j, i, 0] = 255;// bleu clair
									  //  im1.Data[j, i, 1] = 0; im1.Data[j, i, 2] = 0; // bleu fonce
									}
										
								break;
							case 2: // + de 1 graine (>=1)
								for (int i = c.ZoneROI.X + im1.ROI.X; i < c.ZoneROI.Right + im1.ROI.X; i++)
									for (int j = c.ZoneROI.Y + im1.ROI.Y; j < c.ZoneROI.Bottom + im1.ROI.Y; j++)
									{
										im1.Data[j, i, 0] = 0; // jaune fonce
									   // im1.Data[j, i, 1] = 255; im1.Data[j, i, 2] = 255; // jaune clair
									}
								break;
							default:
								for (int i = c.ZoneROI.X + im1.ROI.X; i < c.ZoneROI.Right + im1.ROI.X; i++)
									for (int j = c.ZoneROI.Y + im1.ROI.Y; j < c.ZoneROI.Bottom + im1.ROI.Y; j++)
									{
										im1.Data[j, i, 0] = 0; // jaune fonce
									   // im1.Data[j, i, 1] = 255; im1.Data[j, i, 2] = 255; // jaune clair
									}
								break;

					   // };

						/*  foreach (Graine g in c.ListeGraine)
						  {
							  Point[] s = g.Contour.ToArray();
							  for (int i = 0; i < s.Length; i++)
							  {
								  s[i].X = s[i].X + c.ZoneROI.X;
								  s[i].Y = s[i].Y + c.ZoneROI.Y;
							  }
							  im1.DrawPolyline(s, true, new Bgr(Color.Blue), 3);
						  }*/
					}

				}

				
				/* calculer le nombre de graine 0,1 ou 2 */
				int nbGraine0,nbGraine1,nbGrainePlus;

				List<Recipient> liste = listeContenant.FindAll(delegate(Recipient g) { return g.NombreGraine == 0; });
				nbGraine0 =  liste == null ? 0 : liste.Count;

				liste = listeContenant.FindAll(delegate(Recipient g) { return g.NombreGraine == 1; });
				nbGraine1 =  liste == null ? 0 : liste.Count;

				liste = listeContenant.FindAll(delegate(Recipient g) { return g.NombreGraine > 1; });
				nbGrainePlus =  liste == null ? 0 : liste.Count;


				//  on reduit de 1/2
				im1 = im1.Resize(0.5, Emgu.CV.CvEnum.INTER.CV_INTER_AREA);
				// on retourne l'imageNIR
				im1 = im1.Rotate(90, new Bgr(Color.Blue), false);


				// on ecrit
				MCvFont f = new MCvFont(Emgu.CV.CvEnum.FONT.CV_FONT_HERSHEY_COMPLEX, 1.0, 1.0);
				im1.Draw("NB0->" + nbGraine0.ToString().Trim(), ref f, new Point(100, 380), new Bgr(255, 255, 255));
				im1.Draw("NB>1->" + nbGrainePlus.ToString().Trim(), ref f, new Point(100, 480), new Bgr(255, 255, 255));

				//On retourne imageNIR
				 return im1;
			}
			catch
			{
				System.Console.WriteLine("J'ai plante en fesant des carre");
				return null;
			}
		}

		public double TrouverSeuilOptimalOtsu(Image<Bgr, Byte> im)
		{

			try
			{
				int[] hist = new int[256];

				Image<Gray, Byte> im1 = im.Convert<Gray, Byte>();

				for (int k = 0; k < 256; k++)
				{
					hist[k] = 0;
				}

				for (int i = im1.ROI.Y; i < im1.ROI.Height; i++)
				{
					for (int j = im1.ROI.X; j < im1.ROI.Width; j++)
					{
						int val = im1.Data[i, j, 0];
						hist[val]++;
					}
				}

				double sumT = 0.0;

				for (int t = 0; t < 256; t++)
					sumT += (double)(t) * (double)(hist[t]);

				double sumB = 0.0;
				int wB = 0;
				int wF = 0;

				double varmax = 0.0;
				int tOptimal = 0;

				for (int t = 0; t < 100; t++)
				{
					wB += hist[t];
					if (wB == 0)
						continue;

					wF = im1.Width * im1.Height - wB;
					if (wF == 0)
						break;

					sumB += (double)(t) * (double)(hist[t]);

					double mB = sumB / (double)(wB);
					double mF = (sumT - sumB) / (double)(wF);

					double varbetween = (double)(wB) * (double)(wF) * (mB - mF) * (mB - mF);

					if (varbetween > varmax)
					{
						varmax = varbetween;
						tOptimal = t;
					}
				}


				return tOptimal;

			}
			catch
			{
				System.Console.WriteLine("Incapable de trouver le seuil optimal de Otsu");
				return 0;
			}
		}


		/* -----------------------------------------------------------------------------------------------------
		* Fonction ajoutant dessin
		*  -----------------------------------------------------------------------------------------------------*/

		public Image<Bgr, Byte> dessiner2(Image<Bgr, Byte> im1)
		{
			Int32 nbSansGraine = 0;
			Int32 nbPlusQueUneGraine = 0; 


			try
			{
				foreach (Recipient c in listeContenant)
				{
					if (c.Position == 1)
					{
						switch (c.NombreGraine)
						{
							case 0: // Pas de graine dans la cellule
								for (int i = c.ZoneROI.X + im1.ROI.X; i < c.ZoneROI.Right + im1.ROI.X; i++)
									for (int j = c.ZoneROI.Y + im1.ROI.Y; j < c.ZoneROI.Bottom + im1.ROI.Y; j++)
									{
										im1.Data[j, i, 2] = 255; // rouge clair
										nbSansGraine++;
										//im1.Data[j, i, 0] = 0; im1.Data[j, i, 1] = 0; // rouge fonce
									}
								 break;
							case 1: // Une seule graine BLEU => OK
								for (int i = c.ZoneROI.X + im1.ROI.X; i < c.ZoneROI.Right + im1.ROI.X; i++)
									for (int j = c.ZoneROI.Y + im1.ROI.Y; j < c.ZoneROI.Bottom + im1.ROI.Y; j++)
									{
										im1.Data[j, i, 0] = 255;// bleu clair
									  //  im1.Data[j, i, 1] = 0; im1.Data[j, i, 2] = 0; // bleu fonce
									}
										
								break;
							case 2: // + de 1 graine (>=1)
								for (int i = c.ZoneROI.X + im1.ROI.X; i < c.ZoneROI.Right + im1.ROI.X; i++)
									for (int j = c.ZoneROI.Y + im1.ROI.Y; j < c.ZoneROI.Bottom + im1.ROI.Y; j++)
									{
										im1.Data[j, i, 0] = 0; // jaune fonce
										nbPlusQueUneGraine++;
									   // im1.Data[j, i, 1] = 255; im1.Data[j, i, 2] = 255; // jaune clair
									}
								break;
							default:
								for (int i = c.ZoneROI.X + im1.ROI.X; i < c.ZoneROI.Right + im1.ROI.X; i++)
									for (int j = c.ZoneROI.Y + im1.ROI.Y; j < c.ZoneROI.Bottom + im1.ROI.Y; j++)
									{
										im1.Data[j, i, 0] = 0; // jaune fonce
									   // im1.Data[j, i, 1] = 255; im1.Data[j, i, 2] = 255; // jaune clair
									}
								break;

						};
						/*  foreach (Graine g in c.ListeGraine)
						  {
							  Point[] s = g.Contour.ToArray();
							  for (int i = 0; i < s.Length; i++)
							  {
								  s[i].X = s[i].X + c.ZoneROI.X;
								  s[i].Y = s[i].Y + c.ZoneROI.Y;
							  }
							  im1.DrawPolyline(s, true, new Bgr(Color.Blue), 3);
						  }*/
					}

				}
				StringBuilder ImageComments = new StringBuilder();
				ImageComments.AppendLine("0: " + nbSansGraine);
				ImageComments.AppendLine(">1: " + nbPlusQueUneGraine);
				Point location = new Point(im1.ROI.X+20, im1.ROI.Y+20);
				MCvFont font = new MCvFont(Emgu.CV.CvEnum.FONT.CV_FONT_HERSHEY_COMPLEX, 5.0, 5.0);
				Bgr color = new Bgr(255.0,255.0,255.0);

				Image<Bgr, Byte> imText = new Image<Bgr, byte>("test.jpg");
				//imText = im1.CopyBlank();
			   // imText.ROI = im1.ROI;



			   // imText.Draw(ImageComments.ToString, ref font, location, color);
				string st1 = string.Concat("0: ", nbSansGraine);
				string st2 = string.Concat(">1: ", nbPlusQueUneGraine);
				//string st3 = string.Concat(st1, st2);
				string st3 = "TEST";
				//im1.Draw(st3, ref font, location, color);
			   // imText.Draw("0: ", ref font, location, color);
			  //  Bgr colorFond = new Bgr(255.0,255.0,255.0);
			 //   imText.Rotate(-90.0, colorFond);

			   // im1 = im1.AddWeighted(imText, 0.0, 1.0, 0.0);

				imText.Dispose();
				imText = null;


				return im1;
			}
			catch
			{
				System.Console.WriteLine("J'ai plante en dessinant des carres");
				return null;
			}
		}

		public Image<Gray, Byte> dessinerDiff(Image<Bgr, Byte> image, Image<Bgr, Byte> back)
		{

			Rectangle r = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height);
			image.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight);

			Image<Bgr, Byte> Difference = image.AbsDiff(back);


			//Image<Gray, Byte> DifferenceGray = Difference.Convert<Gray, Byte>();

            Image<Gray, Byte> DifferenceGray = Difference.InRange(new Bgr(0,0,0), new Bgr(5,5,5)); //85
            
			image.ROI = r;
			
			return DifferenceGray;

		}

		public Image<Gray, Byte> dessinerDiff(Image<Bgr, Byte> image, Image<Gray, Byte> back)
		{

			Rectangle r = new Rectangle(image.ROI.X, image.ROI.Y, image.ROI.Width, image.ROI.Height);
			image.ROI = new Rectangle(0, 0, Settings.Default.camWidth, Settings.Default.camHeight);

			Image<Gray, Byte> imageG = image.Convert<Gray, Byte>();

			Image<Gray, Byte> DifferenceGray = imageG.AbsDiff(back);

            //double moy = DetecterMoyenneDifference(DifferenceGray);

			image.ROI = r;

	  

			return DifferenceGray;

		}


		// Public implementation of Dispose pattern callable by consumers. 
		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		// Protected implementation of Dispose pattern. 
		protected virtual void Dispose(bool disposing)
		{
			if (disposed)
				return;

			if (disposing)
			{
				// Free any other managed objects here. 
				//
			}

			// Free any unmanaged objects here. 
			//
			disposed = true;
		}

		/*Determination automatique du facteur de correction à partir de l'imageNIR d'un damier*/
		public static void FacteurCorrection(Image<Bgr, byte> imageDamier)
		{
			/*Convertie imageNIR en noire et blanc + adoucissement de l'imageNIR*/
			Image<Gray, byte> modifiedImage = imageDamier.Convert<Gray, byte>();
			modifiedImage = modifiedImage.SmoothMedian(15);
			modifiedImage._GammaCorrect(0.3d);
            Size imageDamierSize = new Size(imageDamier.Width, imageDamier.Height);

            

			/*Pts sur damier deforme*/
			PointF[] ptD = new PointF[] { };

			/*Dimension du damier*/
			int widthCB = Properties.Settings.Default.widthChessBoard;
			int heightCB = Properties.Settings.Default.heightChessBoard;
			Size patternSize = new Size(widthCB, heightCB);

			/*Intersections du damier deforme*/
			CameraCalibration.FindChessboardCorners(modifiedImage, patternSize, Emgu.CV.CvEnum.CALIB_CB_TYPE.ADAPTIVE_THRESH | Emgu.CV.CvEnum.CALIB_CB_TYPE.FILTER_QUADS, out ptD);

			/*Coordonnees du carre central*/
			int pt1 = widthCB / 2 - 1  + (heightCB / 2 - 1) * widthCB;
			int pt2 = pt1 + 1;
			int pt3 = pt1 + widthCB;
			int pt4 = pt3 + 1;
			float x1 = ptD[pt1].X;
			float y1 = ptD[pt1].Y;
			float x2 = ptD[pt2].X;
			float y2 = ptD[pt2].Y;
			float x3 = ptD[pt3].X;
			float y3 = ptD[pt3].Y;
			float x4 = ptD[pt4].X;
			float y4 = ptD[pt4].Y;

			/*Dimension du carre central*/
			double width_center = (Math.Sqrt(Math.Pow(x2 - x1, 2) + Math.Pow(y2 - y1, 2)) + Math.Sqrt(Math.Pow(x3 - x4, 2) + Math.Pow(y3 - y4, 2))) / 2;
			double height_center = (Math.Sqrt(Math.Pow(y1 - y3, 2) + Math.Pow(x1 - x3, 2)) + Math.Sqrt(Math.Pow(y2 - y4, 2) + Math.Pow(x2 - x4, 2))) / 2;
			double dim = (width_center + height_center) / 2;

			/*Pts theoriques sur imageNIR non-deformee (pts de contrôle)*/
			List<PointF> ptC = new List<PointF>();

			/*for (int j = 0; j < heightCB; j++)
                for (int i = 0; i < widthCB; i++)
                {
                    ptC.Add(new PointF((float)(dim * i) + ptD[0].X, (float)(dim * j) + ptD[0].Y));
                    PointF center = new PointF((float)(dim * i) + ptD[0].X, (float)(dim * j) + ptD[0].Y);
                    CircleF circle = new CircleF(center, 5);
                    imageDamier.Draw(circle, new Bgr(Color.Red), 2);
                }*/

            /*for (int j = -heightCB/2; j <= heightCB/2; j++)
                for (int i = -widthCB/2; i <= widthCB/2; i++)
                {
                    ptC.Add(new PointF((float)(dim * i) + ptD[pt1].X, (float)(dim * j) + ptD[pt1].Y));
                    PointF center = new PointF((float)(dim * i) + ptD[pt1].X, (float)(dim * j) + ptD[pt1].Y);
                    CircleF circle = new CircleF(center, 5);
                    imageDamier.Draw(circle, new Bgr(Color.Red), 2);
                }*/

                int Xc = pt1 % widthCB;
                int Yc = pt1 / widthCB;

                for (int i = 0; i < ptD.Length; i++)
                {
                    ptC.Add(new PointF((float)(dim * ((i % widthCB) - Xc)) + ptD[pt1].X, (float)(dim * ((i / widthCB) - Yc)) + ptD[pt1].Y));
                    PointF center = new PointF((float)(dim * ((i % widthCB) - Xc)) + ptD[pt1].X, (float)(dim * ((i / widthCB) - Yc)) + ptD[pt1].Y);
                    CircleF circle = new CircleF(center, 5);
                    imageDamier.Draw(circle, new Bgr(Color.Red), 2);
                }

       
                for (int i = 0; i < ptD.Length; i++)
                {
                    //ptC.Add(new PointF((float)(dim * i) + ptD[0].X, (float)(dim * j) + ptD[0].Y));
                    PointF center = new PointF((float)ptD[i].X, (float)ptD[i].Y);
                    CircleF circle = new CircleF(center, 5);
                    imageDamier.Draw(circle, new Bgr(Color.Blue), 2);
                }



            imageDamier.Save("imageDamierPositionCarre.jpg");

            


			/*Resoudre système*/
			var x = SolveSystem(ptC, ptD, widthCB, heightCB, imageDamierSize);

            Rectangle r1 = new Rectangle(0, 0, imageDamier.Width, imageDamier.Height);
            Rectangle r2 = new Rectangle(0, 0, imageDamier.Width, imageDamier.Height);

            Image<Bgr, Byte> imageDamierCorriger = new Image<Bgr, Byte>("damier8X10.jpg");

           // imageDamierCorriger = imageDamier.Copy();
            
            //imageDamierCorriger = corrigerImage(imageDamierCorriger, r1, r2);

            imageDamierCorriger = corrigerImageS(imageDamierCorriger, r1, r2);

            imageDamierCorriger.Save("imageDamierCorrigee.jpg");



			/*ecrire facteur de correction dans fichier settings (fichier local propre au User)*/
			try
			{
				Settings.Default.facteur1 = x[0];// 18/04 -3.5040526712117047 X 10-8
				Settings.Default.facteur2 = x[1]; // 18/04 7.72543622516225 X 10-15
				Settings.Default.Save();
			}
			catch (Exception err)
			{
				MessageBox.Show("Impossible de sauvegarder les facteurs de correction : \n" + err.Message,
					"Erreur !",
					MessageBoxButtons.OK,
					MessageBoxIcon.Exclamation,
					MessageBoxDefaultButton.Button1);
			}


		}

		/*Resoudre système avec ax=b*/
		private static Vector<double> SolveSystem(List<PointF> ptC, PointF[] ptD, int widthCB, int heightCB, Size imageSize)
		{
			double[][] a = new double[2 * widthCB * heightCB][];

			double[] b = new double[2 * widthCB * heightCB];

            int decX = imageSize.Width / 2;
            int decY = imageSize.Height / 2;



			for (int i = 0; i < 2 * widthCB * heightCB; i++)
			{
				a[i] = new double[2];
			}

			for (int i = 0; i < widthCB * heightCB; i++)
			{
				//double r = Math.Sqrt((ptC[i].X-decX) * (ptC[i].X-decX) + (ptC[i].Y-decY) * (ptC[i].Y-decY));
                double r2 = (ptC[i].X - decX) * (ptC[i].X - decX) + (ptC[i].Y - decY) * (ptC[i].Y - decY);

				/*a[2 * i][0] = r * r * ptC[i].X;
				a[2 * i][1] = r * r * r * r * ptC[i].X;

				a[2 * i + 1][0] = r * r * ptC[i].Y;
				a[2 * i + 1][1] = r * r * r * r * ptC[i].Y;*/
                a[2 * i][0] = r2 * (ptC[i].X - decX);
                a[2 * i][1] = r2 * r2 * (ptC[i].X - decX);

                a[2 * i + 1][0] = r2 * (ptC[i].Y-decY);
                a[2 * i + 1][1] = r2 * r2 * (ptC[i].Y-decY);

			}

			for (int i = 0; i < widthCB * heightCB; i++)
			{
				b[2 * i] = ptD[i].X - ptC[i].X;
				b[2 * i + 1] = ptD[i].Y - ptC[i].Y;
			}

			var matrixA = DenseMatrix.OfRowArrays(a);
			var vectorB = new DenseVector(b);

			var x = matrixA.Svd().Solve(vectorB);

			return x;
		}
	}
}